/*
	Tx Line - A Transmission Line Calculator
	Copyright © 2007 Harry Whitfield

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
	
	Tx Line - version 1.1
	2 December, 2007
	Copyright © 2007 Harry Whitfield
	mailto:g6auc@arrl.net
*/

		include("menu.js");

		var minWidth = 180;
		var midWidth = 310;
		var maxWidth = 565;
		
		var curWidth  = minWidth;
		var userWidth = minWidth;

		var debug = false;
		
		var redButtonWithin = false;
		var amberButtonWithin = false;
		var greenButtonWithin = false;
				
		var hasFocus = false;
		
		var mode     = Number(preferences.modePref.value);
		var lineType = Number(preferences.linePref.value);

		var paleGreen = "#CCFFCC";
		var white     = "#FFFFFF";
		
		var modeText    = ['Outer Radius','Inner Radius','Relative Permittivity','Characteristic Impedance'];
		var altModeText = ['Line Separation','Line Radius','Relative Permittivity','Characteristic Impedance'];
		
		
		var lineText = ['Coaxial Cable','Parallel Line'];
		var dockText = ['Coaxial Cable','Parallel Line'];

		var modeValue = [14,13,11,7];
		var modeIndex = [-1,-1,-1,-1,-1,-1,-1,3,-1,-1,-1,2,-1,1,0];
		
		var multiplier;

		function parseNonNegFloat(s)
		{
			var v = parseFloat(s);
			if (isNaN(v)) { return v; }
			if (v >= 0)   { return v; }
			return NaN;
		}

		function parseDielecFloat(s)
		{
			var v = parseFloat(s);
			if (isNaN(v)) { return v; }
			if (v >= 1)   { return v; }
			return NaN;
		}
		
		function process()
		{
			var mu0 = 4*Math.PI*1e-7;		// Permeability of Vacuum
			var c   = 299792458;			// Speed of light in Vacuum
			var e0  = 1/(mu0*c*c);			// Permittivity of Vacuum
			var Z0  = c*mu0;				// Characteristic Impedance of Vacuum

			function charImp(o, i, e)	// Characteristic Impedance of Line
			{
				return Z0*Math.log(o/i)/(multiplier*Math.PI*Math.sqrt(e));
			}

			function relPerm(o, i, Z)
			{
				var sqrtE = Z0*Math.log(o/i)/(multiplier*Math.PI*Z);
				return sqrtE*sqrtE;
			}

			function innerRad(o, e, Z)
			{
				return o/Math.exp(Z*multiplier*Math.PI*Math.sqrt(e)/Z0);
			}

			function outerRad(i, e, Z)
			{
				return i*Math.exp(Z*multiplier*Math.PI*Math.sqrt(e)/Z0);
			}

			var outer, inner, e, Z, result; 
			
			switch (mode)	// Z=8, e=4, inner=2, ouuter=1
			{
				case 7: outer  = parseNonNegFloat(outerRadius.data);
						inner  = parseNonNegFloat(innerRadius.data);
						e      = parseDielecFloat(relPermittivity.data);
						result = charImp(outer, inner, e); print('result: ' + result);
						if(isNaN(result)) { result = 'Invalid Input'; }
						impedance.data = result;
						break;
			
				case 11:outer = parseNonNegFloat(outerRadius.data);
						inner = parseNonNegFloat(innerRadius.data);
						Z     = parseNonNegFloat(impedance.data);
						result = relPerm(outer, inner, Z); print('result: ' + result);
						if(isNaN(result)) { result = 'Invalid Input'; }
						relPermittivity.data = result;
						break;
						
				case 13:outer = parseNonNegFloat(outerRadius.data);
						e     = parseDielecFloat(relPermittivity.data);
						Z     = parseNonNegFloat(impedance.data);
						result = innerRad(outer, e, Z); print('result: ' + result);
						if(isNaN(result)) { result = 'Invalid Input'; }
						innerRadius.data = result;
						break;

				case 14:inner = parseNonNegFloat(innerRadius.data);
						e     = parseDielecFloat(relPermittivity.data);
						Z     = parseNonNegFloat(impedance.data);
						result = outerRad(inner, e, Z); print('result: ' + result);
						if(isNaN(result)) { result = 'Invalid Input'; }
						outerRadius.data = result;
						break;

				default:beep();
						break;
			}
		}
		
		function setAmber(signed)
		{
			if (signed)
			{
				amberButton.src = "Resources/amberMinus.png";
			}
			else
			{
				amberButton.src = "Resources/amber.png";
			}
		}

		function normalButtons()
		{
			redButton.src   = "Resources/red.png";
			greenButton.src = "Resources/green.png";
			setAmber(false);
		}

		function activeButtons()
		{
			redButton.src   = "Resources/redCross.png";
			greenButton.src = "Resources/greenPlus.png";
			setAmber(true);
		}

		function focusButtons()
		{
			if (hasFocus)
			{
				redButton.src   = "Resources/red.png";
				greenButton.src = "Resources/green.png";
				setAmber(false);
			}
			else
			{
				redButton.src   = "Resources/grey.png";
				amberButton.src = "Resources/grey.png";
				greenButton.src = "Resources/grey.png";
			}
		}

		function toggleSize()
		{
			//print('main_window.width=' + main_window.width);
			
			if (main_window.width == userWidth)
			{
				setUp(midWidth);
			}
			else
			{
				setUp(userWidth);
			}
			if (network_window.visible)
			{
				network_window.hOffset = main_window.hOffset + main_window.width + 10;
				network_window.vOffset = main_window.vOffset;
			}
		}

		function setUp(width)
		{
			if (debug)
			{
				print('setUp():======== start ========:');
				print('width=' + width);
				print('minWidth=' + minWidth);
				print('maxWidth=' + maxWidth);
			}
			
			if (width < minWidth) { width = minWidth; }
			if (width > maxWidth) { width = maxWidth; }

			main_window.width = width;
			middle.width = width - 40;
			right.hOffset = width - 20;
			
			outerRadius.width = width - 30;
			innerRadius.width = width - 30;
			relPermittivity.width = width - 30;
			impedance.width = width - 30;
						
			dragbox.hOffset = width - 15;
			
			if (network_window.visible)
			{
				network_window.hOffset = main_window.hOffset + main_window.width + 10;
				network_window.vOffset = main_window.vOffset;
			}

			updateNow();
			
			if (debug)
			{
				print('width=' + width);
		
				print('main_window.width=' + main_window.width);
				print('middle.width=' + middle.width);
				print('right.hOffset=' + right.hOffset);
			
				print('outerRadius.width=' + outerRadius.width);
				print('innerRadius.width=' + innerRadius.width);
				print('relPermittivity.width=' + relPermittivity.width);
				print('impedance.width=' + impedance.width);
				
				print('dragbox.hOffset=' + dragbox.hOffset);

				print('setUp():========  end  ========:');
			}

			return width;
		}
		
		function clearData()
		{
			outerRadius.data = "";
			innerRadius.data = "";
			relPermittivity.data = "";
			impedance.data = "";
		}
		
		function showDockImage()
		{
			if (lineType === 0)	// Co-ax Cable
			{
				buildVitality("Resources/coax.png",  224, 200, dockText[lineType]);
			}
			else
			{
				buildVitality("Resources/line.png", 224, 200, dockText[lineType]);
			}
		}

		function updatePrefs()
		{
			mode = Number(preferences.modePref.value);
			chooseMenu.setTick(modeText[modeIndex[mode]]);

			lineType = Number(preferences.linePref.value);
			inputMenu.setTick(lineText[lineType]);
			
			if (lineType === 0)	// coax
			{
				multiplier = 2.0;
				outerRadiusHeading.data = "Outer Radius";
				innerRadiusHeading.data = "Inner Radius";
				outerRadius.tooltip = "Outer radius of the coaxial cable (i.e. the radius of the inner surface of the shield).";
				innerRadius.tooltip = "Inner radius of the coaxial cable (i.e. the radius of the outer surface of the central conductor).";
				chooseMenu.reset(modeText, modeText[modeIndex[mode]]);
				preferences.modePref.option = modeText;
				buildVitality("Resources/coax.png",  224, 200, dockText[lineType]);
			}
			else
			{
				multiplier = 1.0;
				outerRadiusHeading.data = "Line Separation";
				innerRadiusHeading.data = "Line Radius";
				outerRadius.tooltip = "Line Separation (i.e. the distance between the centers of the conductors).";
				innerRadius.tooltip = "Line Radius (i.e. the radius of the outer surface of each conductor).";
				chooseMenu.reset(altModeText, altModeText[modeIndex[mode]]);
				preferences.modePref.option = altModeText;
				buildVitality("Resources/line.png", 224, 200, dockText[lineType]);
			}

			userWidth = curWidth;
			
			outerRadius.font = innerRadius.font = relPermittivity.font = impedance.font =  preferences.generalFontPref.value;
			paleGreen = preferences.bgColorPref.value;
			
			outerRadius.editable     = Boolean(mode & 1); if (mode & 1) { outerRadius.bgColor     = paleGreen; } else { outerRadius.bgColor     = white; }
			innerRadius.editable     = Boolean(mode & 2); if (mode & 2) { innerRadius.bgColor     = paleGreen; } else { innerRadius.bgColor     = white; }
			relPermittivity.editable = Boolean(mode & 4); if (mode & 4) { relPermittivity.bgColor = paleGreen; } else { relPermittivity.bgColor = white; }
			impedance.editable       = Boolean(mode & 8); if (mode & 8) { impedance.bgColor       = paleGreen; } else { impedance.bgColor       = white; }
			
			clearData();
			
			if (mode & 1)
			{
				outerRadius.focus();
				outerRadius.select(0, -1);
			}
			else if (mode & 2)
			{
				innerRadius.focus();
				innerRadius.select(0, -1);
			}
			else if (mode & 4)
			{
				relPermittivity.focus();
				relPermittivity.select(0, -1);
			}
			else if (mode & 8)
			{
				impedance.focus();
				impedance.select(0, -1);
			}

			setAmber(false);
			setUp(curWidth);
		}
		
		function chooseMenuAction()								//	(i, data)	// define the action function for chooseMenu
		{
    		var i = this.i, data = this.data;
    		//print("Index:" + i + " Data:" + data);
    		preferences.modePref.value = String(modeValue[i]);
    		updatePrefs();
		}

		function inputMenuAction()								//	(i, data)	// define the action function for inputMenu
		{
    		var i = this.i, data = this.data;
    		//print("Index:" + i + " Data:" + data);
    		preferences.linePref.value = String(i);
    		updatePrefs();
		}

		var menuStringsArray = modeText; 						// define the array of menuStrings
		var selectedString   = modeText[modeIndex[mode]];  		// define the initially selected string, or set to ""
		var menuActionFunction   = chooseMenuAction;  			// define the name of the menuAction function - must be a function
		var menuImage = amberButton;

		var chooseMenu = new Menu(menuImage, menuStringsArray, selectedString, menuActionFunction); // call the menu constructor

		menuStringsArray = lineText; 					// define the array of menuStrings
		selectedString   = lineText[lineType];  		// define the initially selected string, or set to ""
		menuActionFunction   = inputMenuAction;  		// define the name of the menuAction function - must be a function
		menuImage = greenButton;

		var inputMenu = new Menu(menuImage, menuStringsArray, selectedString, menuActionFunction); // call the menu constructor
		updatePrefs();
		setUp(curWidth);
		focusWidget();
